@@ -12,7 +12,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclCmdMZ.c 1.104 97/10/31 13:06:19
+ * RCS: @(#) $Id: tclCmdMZ.c,v 1.2 1998/09/14 18:39:57 stanton Exp $
  */
 
 #include "tclInt.h"
@@ -1184,7 +1184,7 @@ Tcl_StringObjCmd(dummy, interp, objc, objv)
 	"compare",	"first",	"index",	"last",
 	"length",	"match",	"range",	"tolower",
 	"toupper",	"trim",		"trimleft",	"trimright",
-	"wordend",	"wordstart",	NULL
+	"wordend",	"wordstart",	(char *) NULL
     };
     enum options {
 	STR_COMPARE,	STR_FIRST,	STR_INDEX,	STR_LAST,
@@ -2187,8 +2187,8 @@ TraceVarProc(clientData, interp, name1, name2, flags)
 	 * the command. We discard any object result the command returns.
 	 */
 
-	dummy.objResultPtr = Tcl_NewObj();
-	Tcl_IncrRefCount(dummy.objResultPtr);
+	dummy.objResultPtr1 = Tcl_NewObj();
+	Tcl_IncrRefCount(dummy.objResultPtr1);
 	if (interp->freeProc == 0) {
 	    dummy.freeProc = (Tcl_FreeProc *) 0;
 	    dummy.result = "";
@@ -2223,12 +2223,12 @@ TraceVarProc(clientData, interp, name1, name2, flags)
 	 * Restore the interpreter's object result from saveObjPtr.
 	 */
 
-	oldObjResultPtr = iPtr->objResultPtr;
-	iPtr->objResultPtr = saveObjPtr;  /* was incremented above */
+	oldObjResultPtr = iPtr->objResultPtr1;
+	iPtr->objResultPtr1 = saveObjPtr;  /* was incremented above */
 	Tcl_DecrRefCount(oldObjResultPtr);
 
-	Tcl_DecrRefCount(dummy.objResultPtr);
-	dummy.objResultPtr = NULL;
+	Tcl_DecrRefCount(dummy.objResultPtr1);
+	dummy.objResultPtr1 = NULL;
 	Tcl_DStringFree(&cmd);
     }
     if (flags & TCL_TRACE_DESTROYED) {
