@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclObj.c 1.47 97/10/30 13:39:00
+ * RCS: @(#) $Id: tclObj.c,v 1.3 1998/10/05 22:32:09 escoffon Exp $
  */
 
 #include "tclInt.h"
@@ -134,6 +134,7 @@ InitTypeTable()
     Tcl_RegisterObjType(&tclStringType);
     Tcl_RegisterObjType(&tclListType);
     Tcl_RegisterObjType(&tclByteCodeType);
+    Tcl_RegisterObjType(&tclProcBodyType);
 
     tclEmptyStringRep = (char *) ckalloc((unsigned) 1);
     tclEmptyStringRep[0] = '\0';
@@ -345,7 +346,7 @@ Tcl_AppendAllObjTypes(interp, objPtr)
 
 Tcl_ObjType *
 Tcl_GetObjType(typeName)
-    char *typeName;		/* Name of Tcl object type to look up. */
+    CONST char *typeName;	/* Name of Tcl object type to look up. */
 {
     register Tcl_HashEntry *hPtr;
     Tcl_ObjType *typePtr;
@@ -495,7 +496,7 @@ Tcl_NewObj()
 
 Tcl_Obj *
 Tcl_DbNewObj(file, line)
-    register char *file;	/* The name of the source file calling this
+    register CONST char *file;	/* The name of the source file calling this
 				 * procedure; used for debugging. */
     register int line;		/* Line number in the source file; used
 				 * for debugging. */
@@ -522,7 +523,7 @@ Tcl_DbNewObj(file, line)
 
 Tcl_Obj *
 Tcl_DbNewObj(file, line)
-    char *file;			/* The name of the source file calling this
+    CONST char *file;		/* The name of the source file calling this
 				 * procedure; used for debugging. */
     int line;			/* Line number in the source file; used
 				 * for debugging. */
@@ -698,7 +699,7 @@ Tcl_DuplicateObj(objPtr)
 /*
  *----------------------------------------------------------------------
  *
- * Tcl_GetStringFromObj --
+ * Tcl_GetByteArrayFromObj, Tcl_GetStringFromObj --
  *
  *	Returns the string representation's byte array pointer and length
  *	for an object.
@@ -718,6 +719,30 @@ Tcl_DuplicateObj(objPtr)
  *----------------------------------------------------------------------
  */
 
+#undef Tcl_GetString
+
+char *
+Tcl_GetString(objPtr)
+    register Tcl_Obj *objPtr;
+{
+    if (objPtr->bytes == NULL) {
+	objPtr->typePtr->updateStringProc(objPtr);
+    }
+    return objPtr->bytes;
+
+}
+
+unsigned char *
+Tcl_GetByteArrayFromObj(objPtr, lengthPtr)
+    register Tcl_Obj *objPtr;	/* Object whose string rep byte pointer
+				 * should be returned. */
+    register int *lengthPtr;	/* If non-NULL, the location where the
+				 * string rep's byte array length should be
+				 * stored. If NULL, no length is stored. */
+{
+    return (unsigned char *) Tcl_GetStringFromObj(objPtr, lengthPtr);
+}
+
 char *
 Tcl_GetStringFromObj(objPtr, lengthPtr)
     register Tcl_Obj *objPtr;	/* Object whose string rep byte pointer
@@ -726,14 +751,9 @@ Tcl_GetStringFromObj(objPtr, lengthPtr)
 				 * string rep's byte array length should be
 				 * stored. If NULL, no length is stored. */
 {
-    if (objPtr->bytes != NULL) {
-	if (lengthPtr != NULL) {
-	    *lengthPtr = objPtr->length;
-	}
-	return objPtr->bytes;
+    if (objPtr->bytes == NULL) {
+	objPtr->typePtr->updateStringProc(objPtr);
     }
-
-    objPtr->typePtr->updateStringProc(objPtr);
     if (lengthPtr != NULL) {
 	*lengthPtr = objPtr->length;
     }
@@ -852,7 +872,7 @@ Tcl_NewBooleanObj(boolValue)
 Tcl_Obj *
 Tcl_DbNewBooleanObj(boolValue, file, line)
     register int boolValue;	/* Boolean used to initialize new object. */
-    char *file;			/* The name of the source file calling this
+    CONST char *file;		/* The name of the source file calling this
 				 * procedure; used for debugging. */
     int line;			/* Line number in the source file; used
 				 * for debugging. */
@@ -872,7 +892,7 @@ Tcl_DbNewBooleanObj(boolValue, file, line)
 Tcl_Obj *
 Tcl_DbNewBooleanObj(boolValue, file, line)
     register int boolValue;	/* Boolean used to initialize new object. */
-    char *file;			/* The name of the source file calling this
+    CONST char *file;		/* The name of the source file calling this
 				 * procedure; used for debugging. */
     int line;			/* Line number in the source file; used
 				 * for debugging. */
@@ -1228,7 +1248,7 @@ Tcl_NewDoubleObj(dblValue)
 Tcl_Obj *
 Tcl_DbNewDoubleObj(dblValue, file, line)
     register double dblValue;	/* Double used to initialize the object. */
-    char *file;			/* The name of the source file calling this
+    CONST char *file;		/* The name of the source file calling this
 				 * procedure; used for debugging. */
     int line;			/* Line number in the source file; used
 				 * for debugging. */
@@ -1248,7 +1268,7 @@ Tcl_DbNewDoubleObj(dblValue, file, line)
 Tcl_Obj *
 Tcl_DbNewDoubleObj(dblValue, file, line)
     register double dblValue;	/* Double used to initialize the object. */
-    char *file;			/* The name of the source file calling this
+    CONST char *file;		/* The name of the source file calling this
 				 * procedure; used for debugging. */
     int line;			/* Line number in the source file; used
 				 * for debugging. */
@@ -1911,7 +1931,7 @@ Tcl_Obj *
 Tcl_DbNewLongObj(longValue, file, line)
     register long longValue;	/* Long integer used to initialize the
 				 * new object. */
-    char *file;			/* The name of the source file calling this
+    CONST char *file;		/* The name of the source file calling this
 				 * procedure; used for debugging. */
     int line;			/* Line number in the source file; used
 				 * for debugging. */
@@ -1932,7 +1952,7 @@ Tcl_Obj *
 Tcl_DbNewLongObj(longValue, file, line)
     register long longValue;	/* Long integer used to initialize the
 				 * new object. */
-    char *file;			/* The name of the source file calling this
+    CONST char *file;		/* The name of the source file calling this
 				 * procedure; used for debugging. */
     int line;			/* Line number in the source file; used
 				 * for debugging. */
@@ -2044,7 +2064,7 @@ Tcl_GetLongFromObj(interp, objPtr, longPtr)
 void
 Tcl_DbIncrRefCount(objPtr, file, line)
     register Tcl_Obj *objPtr;	/* The object we are adding a reference to. */
-    char *file;			/* The name of the source file calling this
+    CONST char *file;		/* The name of the source file calling this
 				 * procedure; used for debugging. */
     int line;			/* Line number in the source file; used
 				 * for debugging. */
@@ -2066,9 +2086,9 @@ Tcl_DbIncrRefCount(objPtr, file, line)
  *
  *	This procedure is normally called when debugging: i.e., when
  *	TCL_MEM_DEBUG is defined. This checks to see whether or not
- *	the memory has been freed before decrementing the ref count.
+ *	the memory has been freed before incrementing the ref count.
  *
- *	When TCL_MEM_DEBUG is not defined, this procedure just decrements
+ *	When TCL_MEM_DEBUG is not defined, this procedure just increments
  *	the reference count of the object.
  *
  * Results:
@@ -2083,7 +2103,7 @@ Tcl_DbIncrRefCount(objPtr, file, line)
 void
 Tcl_DbDecrRefCount(objPtr, file, line)
     register Tcl_Obj *objPtr;	/* The object we are adding a reference to. */
-    char *file;			/* The name of the source file calling this
+    CONST char *file;		/* The name of the source file calling this
 				 * procedure; used for debugging. */
     int line;			/* Line number in the source file; used
 				 * for debugging. */
@@ -2107,16 +2127,17 @@ Tcl_DbDecrRefCount(objPtr, file, line)
  *
  *	This procedure is normally called when debugging: i.e., when
  *	TCL_MEM_DEBUG is defined. This checks to see whether or not
- *	the memory has been freed before checking if the object is shared.
+ *	the memory has been freed before incrementing the ref count.
  *
- *	When TCL_MEM_DEBUG is not defined, this procedure just does checks
- *	if the object is shared without checking if it actually exists.
+ *	When TCL_MEM_DEBUG is not defined, this procedure just decrements
+ *	the reference count of the object and throws it away if the count
+ *	is 0 or less.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	None.
+ *	The object's ref count is incremented.
  *
  *----------------------------------------------------------------------
  */
@@ -2124,7 +2145,7 @@ Tcl_DbDecrRefCount(objPtr, file, line)
 int
 Tcl_DbIsShared(objPtr, file, line)
     register Tcl_Obj *objPtr;	/* The object we are adding a reference to. */
-    char *file;			/* The name of the source file calling this
+    CONST char *file;		/* The name of the source file calling this
 				 * procedure; used for debugging. */
     int line;			/* Line number in the source file; used
 				 * for debugging. */
