@@ -11,15 +11,37 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclTestObj.c 1.27 97/05/19 17:37:31
+ * RCS: @(#) $Id: tclTestObj.c,v 1.2 1998/09/14 18:40:02 stanton Exp $
  */
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*
+ * Allow this file to be compiled as loadable library (dll) separate
+ * from the Tcl shared library (or dll). We don't want all
+ * dll symbols to be exported, only the Objtest_Init() function.
+ */
+#undef BUILD_tcl
+
 #include "tclInt.h"
 
+#ifdef TCL_USE_STUBS
+Tcl_StubTable *tclStubTable;
+#else
+#define Tcl_Required(a,b,c) Tcl_PkgRequire(a,"Tcl",b,c)
+#endif
+
+/*
+ * TCL_STORAGE_CLASS is set unconditionally to DLLEXPORT because the
+ * Objtest_Init declaration is in the source file itself, which is only
+ * accessed when we are building a library.
+ */
+
+#undef TCL_STORAGE_CLASS
+#define TCL_STORAGE_CLASS DLLEXPORT
+
 #ifdef __cplusplus
 }
 #define dummy /* */
@@ -38,21 +60,7 @@ static Tcl_Obj *varPtr[NUMBER_OF_OBJECT_VARS];
  * Forward declarations for procedures defined later in this file:
  */
 
-#ifdef __WIN32__
-#if defined(_MSC_VER)
-#   define EXPORT(a,b) __declspec(dllexport) a b
-#else
-#   if defined(__BORLANDC__)
-#	define EXPORT(a,b) a _export b
-#   endif
-#endif
-#endif
-
-#ifndef EXPORT
-#   define EXPORT(a,b) a b
-#endif
-
-EXTERN EXPORT(int,Objtest_Init) _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN Objtest_Init _ANSI_ARGS_((Tcl_Interp *interp));
 
 static int		CheckIfVarUnset _ANSI_ARGS_((Tcl_Interp *interp,
 			    int varIndex));
@@ -111,7 +119,7 @@ Objtest_Init(interp)
 {
     register int i;
     
-    if (Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 1) == NULL) {
+    if (Tcl_Required(interp, TCL_VERSION, 1) == NULL) {
 	return TCL_ERROR;
     }
     for (i = 0;  i < NUMBER_OF_OBJECT_VARS;  i++) {
@@ -544,7 +552,7 @@ TestindexobjCmd(dummy, interp, objc, objv)
 	argv[i-4] = Tcl_GetStringFromObj(objv[i], &length);
     }
     argv[objc-4] = NULL;
-    result = Tcl_GetIndexFromObj(setError ? interp : NULL, objv[3],
+    result = Tcl_GetIndexFromObj(setError ? interp : (Tcl_Interp *) NULL, objv[3],
 	    argv, "token", allowAbbrev ? 0 : TCL_EXACT, &index);
     ckfree((char *) argv);
     if (result == TCL_OK) {
