@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- *  SCCS: @(#) tclUtil.c 1.2 98/06/12 16:49:08
+ *  RCS: @(#) $Id: tclUtil.c,v 1.3 1998/09/14 18:40:02 stanton Exp $
  */
 
 #include "tclInt.h"
@@ -400,7 +400,7 @@ Tcl_SplitList(interp, list, argcPtr, argvPtr)
     CONST char *list;		/* Pointer to string with list structure. */
     int *argcPtr;		/* Pointer to location to fill in with
 				 * the number of elements in the list. */
-    char ***argvPtr;		/* Pointer to place to store pointer to
+    char **argvPtr[];		/* Pointer to place to store pointer to
 				 * array of pointers to list elements. */
 {
     char **argv;
@@ -488,6 +488,8 @@ Tcl_SplitList(interp, list, argcPtr, argvPtr)
  *----------------------------------------------------------------------
  */
 
+#undef Tcl_ScanElement
+
 int
 Tcl_ScanElement(string, flagPtr)
     CONST char *string;		/* String to convert to Tcl list element. */
@@ -656,6 +658,8 @@ Tcl_ScanCountedElement(string, length, flagPtr)
  *----------------------------------------------------------------------
  */
 
+#undef Tcl_ConvertElement
+
 int
 Tcl_ConvertElement(src, dst, flags)
     CONST char *src;		/* Source information for list element. */
@@ -827,7 +831,7 @@ Tcl_ConvertCountedElement(src, length, dst, flags)
 char *
 Tcl_Merge(argc, argv)
     int argc;			/* How many strings to merge. */
-    char **argv;		/* Array of string values. */
+    char *argv[];		/* Array of string values. */
 {
 #   define LOCAL_SIZE 20
     int localFlags[LOCAL_SIZE], *flagPtr;
@@ -1322,9 +1326,9 @@ Tcl_SetObjResult(interp, objPtr)
 				 * object. */
 {
     Interp *iPtr = (Interp *) interp;
-    Tcl_Obj *oldObjResult = iPtr->objResultPtr;
+    Tcl_Obj *oldObjResult = iPtr->objResultPtr1;
 
-    iPtr->objResultPtr = objPtr;
+    iPtr->objResultPtr1 = objPtr;
     Tcl_IncrRefCount(objPtr);	/* since interp result is a reference */
 
     /*
@@ -1388,7 +1392,7 @@ Tcl_GetObjResult(interp)
     if (*(iPtr->result) != 0) {
 	TclResetObjResult(iPtr);
 	
-	objResultPtr = iPtr->objResultPtr;
+	objResultPtr = iPtr->objResultPtr1;
 	length = strlen(iPtr->result);
 	TclInitStringRep(objResultPtr, iPtr->result, length);
 	
@@ -1404,7 +1408,7 @@ Tcl_GetObjResult(interp)
 	iPtr->result = iPtr->resultSpace;
 	iPtr->resultSpace[0] = 0;
     }
-    return iPtr->objResultPtr;
+    return iPtr->objResultPtr1;
 }
 
 /*
@@ -2470,6 +2474,14 @@ Tcl_PrintDouble(interp, value, dst)
 
     sprintf(dst, precisionFormat, value);
 
+#ifdef _MSC_VER
+    p = strchr(dst,'e');
+    if (p && p[2] == '0' && strlen(p) == 5) {
+        p[2] = p[3];
+        p[3] = p[4];
+       	p[4] = 0;
+    }
+#endif
     /*
      * If the ASCII result looks like an integer, add ".0" so that it
      * doesn't look like an integer anymore.  This prevents floating-point
@@ -2842,3 +2854,516 @@ Tcl_GetNameOfExecutable()
 {
     return (tclExecutableName);
 }
+
+/*
+ *-------------------------------------------------------------------------
+ *
+ * Tcl_ExternalToUtfDString --
+ *
+ *	Convert a source buffer from the specified encoding into UTF-8.
+ *	If any of the bytes in the source buffer are invalid or cannot
+ *	be represented in the target encoding, a default fallback
+ *	character will be substituted.
+ *
+ * Results:
+ *	The converted bytes are stored in the DString, which is then NULL
+ *	terminated.  The return value is a pointer to the value stored 
+ *	in the DString.
+ *
+ * Side effects:
+ *	None.
+ *
+ *-------------------------------------------------------------------------
+ */
+
+char * 
+Tcl_ExternalToUtfDString(encoding, src, srcLen, dstPtr)
+    VOID *encoding;		/* The encoding for the source string, or
+				 * NULL for the default system encoding. */
+    CONST char *src;		/* Source string in specified encoding. */
+    int srcLen;			/* Source string length in bytes, or < 0 for
+				 * encoding-specific string length. */
+    Tcl_DString *dstPtr;	/* Uninitialized or free DString in which 
+				 * the converted string is stored. */
+{
+    Tcl_DStringInit(dstPtr);
+    Tcl_DStringAppend(dstPtr, src, srcLen);
+    return Tcl_DStringValue(dstPtr);
+}
+
+/*
+ *-------------------------------------------------------------------------
+ *
+ * Tcl_ExternalToUtf --
+ *
+ *	Convert a source buffer from the specified encoding into UTF-8,
+ *
+ * Results:
+ *	The return value is one of TCL_OK, TCL_CONVERT_MULTIBYTE,
+ *	TCL_CONVERT_SYNTAX, TCL_CONVERT_UNKNOWN, or TCL_CONVERT_NOSPACE,
+ *	as documented in tcl.h.
+ *
+ * Side effects:
+ *	The converted bytes are stored in the output buffer.  
+ *
+ *-------------------------------------------------------------------------
+ */
+
+int
+Tcl_ExternalToUtf(interp, encoding, src, srcLen, flags, statePtr, dst,
+	dstLen, srcReadPtr, dstWrotePtr, dstCharsPtr)
+    Tcl_Interp *interp;		/* Interp for error return, if not NULL. */
+    VOID *encoding;		/* The encoding for the source string, or
+				 * NULL for the default system encoding. */
+    CONST char *src;		/* Source string in specified encoding. */
+    int srcLen;			/* Source string length in bytes, or < 0 for
+				 * encoding-specific string length. */
+    int flags;			/* Conversion control flags. */
+    VOID *statePtr;		/* Place for conversion routine to store
+				 * state information used during a piecewise
+				 * conversion.  Contents of statePtr are
+				 * initialized and/or reset by conversion
+				 * routine under control of flags argument. */
+    char *dst;			/* Output buffer in which converted string
+				 * is stored. */
+    int dstLen;			/* The maximum length of output buffer in
+				 * bytes. */
+    int *srcReadPtr;		/* Filled with the number of bytes from the
+				 * source string that were converted.  This
+				 * may be less than the original source length
+				 * if there was a problem converting some
+				 * source characters. */
+    int *dstWrotePtr;		/* Filled with the number of bytes that were
+				 * stored in the output buffer as a result of
+				 * the conversion. */
+    int *dstCharsPtr;		/* Filled with the number of characters that
+				 * correspond to the bytes stored in the
+				 * output buffer. */
+{
+    int exitCode = TCL_OK;
+    if (src == NULL) {
+	srcLen = 0;
+    } else if (srcLen < 0) {
+	srcLen = strlen(src);
+    }
+    if (srcLen > dstLen) {
+	exitCode = TCL_CONVERT_NOSPACE;
+	srcLen = dstLen;
+    }
+    memcpy(dst, src, srcLen);
+    if (srcReadPtr) {
+	*srcReadPtr = srcLen;
+    }
+    if (dstWrotePtr) {
+	*dstWrotePtr = srcLen;
+    }
+    if (dstCharsPtr) {
+	*dstCharsPtr = srcLen;
+    }
+    return exitCode;
+}
+
+/*
+ *-------------------------------------------------------------------------
+ *
+ * Tcl_UtfToExternalDString --
+ *
+ *	Convert a source buffer from UTF-8 into the specified encoding.
+ *	If any of the bytes in the source buffer are invalid or cannot
+ *	be represented in the target encoding, a default fallback
+ *	character will be substituted.
+ *
+ * Results:
+ *	The converted bytes are stored in the DString, which is then
+ *	NULL terminated in an encoding-specific manner.  The return value 
+ *	is a pointer to the value stored in the DString.
+ *
+ * Side effects:
+ *	None.
+ *
+ *-------------------------------------------------------------------------
+ */
+
+char *
+Tcl_UtfToExternalDString(encoding, src, srcLen, dstPtr)
+    VOID *encoding;		/* The encoding for the converted string,
+				 * or NULL for the default system encoding. */
+    CONST char *src;		/* Source string in UTF-8. */
+    int srcLen;			/* Source string length in bytes, or < 0 for
+				 * strlen(). */
+    Tcl_DString *dstPtr;	/* Uninitialized or free DString in which 
+				 * the converted string is stored. */
+{
+    Tcl_DStringInit(dstPtr);
+    Tcl_DStringAppend(dstPtr, src, srcLen);
+    return Tcl_DStringValue(dstPtr);
+}
+
+/*
+ *-------------------------------------------------------------------------
+ *
+ * Tcl_UtfToExternal --
+ *
+ *	Convert a buffer from UTF-8 into the specified encoding.
+ *
+ * Results:
+ *	The return value is one of TCL_OK, TCL_CONVERT_MULTIBYTE,
+ *	TCL_CONVERT_SYNTAX, TCL_CONVERT_UNKNOWN, or TCL_CONVERT_NOSPACE,
+ *	as documented in tcl.h.
+ *
+ * Side effects:
+ *	The converted bytes are stored in the output buffer.  
+ *
+ *-------------------------------------------------------------------------
+ */
+
+int
+Tcl_UtfToExternal(interp, encoding, src, srcLen, flags, statePtr, dst,
+	dstLen, srcReadPtr, dstWrotePtr, dstCharsPtr)
+    Tcl_Interp *interp;		/* Interp for error return, if not NULL. */
+    VOID *encoding;		/* dummy. */
+    CONST char *src;		/* Source string in UTF-8. */
+    int srcLen;			/* Source string length in bytes, or < 0 for
+				 * strlen(). */
+    int flags;			/* Conversion control flags. */
+    VOID *statePtr;		/* Place for conversion routine to store
+				 * state information used during a piecewise
+				 * conversion.  Contents of statePtr are
+				 * initialized and/or reset by conversion
+				 * routine under control of flags argument. */
+    char *dst;			/* Output buffer in which converted string
+				 * is stored. */
+    int dstLen;			/* The maximum length of output buffer in
+				 * bytes. */
+    int *srcReadPtr;		/* Filled with the number of bytes from the
+				 * source string that were converted.  This
+				 * may be less than the original source length
+				 * if there was a problem converting some
+				 * source characters. */
+    int *dstWrotePtr;		/* Filled with the number of bytes that were
+				 * stored in the output buffer as a result of
+				 * the conversion. */
+    int *dstCharsPtr;		/* Filled with the number of characters that
+				 * correspond to the bytes stored in the
+				 * output buffer. */
+{
+    int exitCode = TCL_OK;
+    if (src == NULL) {
+	srcLen = 0;
+    } else if (srcLen < 0) {
+	srcLen = strlen(src);
+    }
+    if (srcLen > dstLen) {
+	exitCode = TCL_CONVERT_NOSPACE;
+	srcLen = dstLen;
+    }
+    memcpy(dst, src, srcLen);
+    if (srcReadPtr) {
+	*srcReadPtr = srcLen;
+    }
+    if (dstWrotePtr) {
+	*dstWrotePtr = srcLen;
+    }
+    if (dstCharsPtr) {
+	*dstCharsPtr = srcLen;
+    }
+    return exitCode;
+}
+
+/*
+ *---------------------------------------------------------------------------
+ *
+ * Tcl_UtfCharComplete --
+ *
+ *	Determine if the UTF-8 string of the given length is long enough
+ *	to be decoded by Tcl_UtfToUniChar().  This does not ensure that the
+ *	UTF-8 string is properly formed.  Equivalent to Plan 9 fullrune().
+ *
+ * Results:
+ *	The return value is 0 if the string is not long enough, non-zero
+ *	otherwise.
+ *
+ * Side effects:
+ *	None.
+ *
+ *---------------------------------------------------------------------------
+ */
+
+int
+Tcl_UtfCharComplete(str, len)
+    CONST char *str;		/* String to check if first few bytes
+				 * contain a complete UTF-8 character. */
+    int len;			/* Length of above string in bytes. */
+{
+    return len >= 1;
+}
+
+/*
+ *---------------------------------------------------------------------------
+ *
+ * Tcl_NumUtfChars --
+ *
+ *	Returns the number of characters (not bytes) in the UTF-8 string,
+ *	not including the terminating NULL byte.  This is equivalent to
+ *	Plan 9 utflen() and utfnlen().
+ *
+ * Results:
+ *	As above.  
+ *
+ * Side effects:
+ *	None.
+ *
+ *---------------------------------------------------------------------------
+ */
+ 
+int 
+Tcl_NumUtfChars(str, len)
+    register CONST char *str;	/* The UTF-8 string to measure. */
+    int len;			/* The length of the string in bytes, or -1
+				 * for strlen(string). */
+{
+    if (len < 0) {
+	len = strlen(str);
+    }
+    return len;
+}
+
+/*
+ *---------------------------------------------------------------------------
+ *
+ * Tcl_UtfFindFirst --
+ *
+ *	Returns a pointer to the first occurance of the given Tcl_UniChar
+ *	in the NULL-terminated UTF-8 string.  The NULL terminator is
+ *	considered part of the UTF-8 string.  Equivalent to Plan 9
+ *	utfrune().
+ *
+ * Results:
+ *	As above.  If the Tcl_UniChar does not exist in the given string,
+ *	the return value is NULL.
+ *
+ * Side effects:
+ *	None.
+ *
+ *---------------------------------------------------------------------------
+ */
+char *
+Tcl_UtfFindFirst(string, ch)
+    CONST char *string;		/* The UTF-8 string to be searched. */
+    int ch;			/* The Tcl_UniChar to search for. */
+{
+    if (ch < 0 || ch > 255) {
+	return NULL;
+    }
+    return strchr(string, ch);
+}
+
+/*
+ *---------------------------------------------------------------------------
+ *
+ * Tcl_UtfFindLast --
+ *
+ *	Returns a pointer to the last occurance of the given Tcl_UniChar
+ *	in the NULL-terminated UTF-8 string.  The NULL terminator is
+ *	considered part of the UTF-8 string.  Equivalent to Plan 9
+ *	utfrrune().
+ *
+ * Results:
+ *	As above.  If the Tcl_UniChar does not exist in the given string,
+ *	the return value is NULL.
+ *
+ * Side effects:
+ *	None.
+ *
+ *---------------------------------------------------------------------------
+ */
+
+char *
+Tcl_UtfFindLast(string, ch)
+    CONST char *string;		/* The UTF-8 string to be searched. */
+    int ch;			/* The Tcl_UniChar to search for. */
+{
+    if (ch < 0 || ch > 255) {
+	return NULL;
+    }
+    return strrchr(string, ch);
+}
+
+/*
+ *---------------------------------------------------------------------------
+ *
+ * Tcl_UtfNext --
+ *
+ *	Given a pointer to some current location in a UTF-8 string,
+ *	move forward one character.  The caller must ensure that they
+ *	are not asking for the next character after the last character
+ *	in the string.
+ *
+ * Results:
+ *	The return value is the pointer to the next character in
+ *	the UTF-8 string.
+ *
+ * Side effects:
+ *	None.
+ *
+ *---------------------------------------------------------------------------
+ */
+ 
+char *
+Tcl_UtfNext(str) 
+    CONST char *str;		    /* The current location in the string. */
+{
+    return (char *) ++str;
+}
+
+/*
+ *---------------------------------------------------------------------------
+ *
+ * Tcl_UtfPrev --
+ *
+ *	Given a pointer to some current location in a UTF-8 string,
+ *	move backwards one character.
+ *
+ * Results:
+ *	The return value is a pointer to the previous character in the
+ *	UTF-8 string.  If the current location was already at the
+ *	beginning of the string, the return value will also be a
+ *	pointer to the beginning of the string.
+ *
+ * Side effects:
+ *	None.
+ *
+ *---------------------------------------------------------------------------
+ */
+
+char *
+Tcl_UtfPrev(str, start)
+    CONST char *str;		    /* The current location in the string. */
+    CONST char *start;		    /* Pointer to the beginning of the
+				     * string, to avoid going backwards too
+				     * far. */
+{
+    if (str > start) {
+	str--;
+    }
+    return (char *) str;
+}
+
+/*
+ *---------------------------------------------------------------------------
+ *
+ * Tcl_UtfAtIndex --
+ *
+ *	Returns a pointer to the specified character (not byte) position
+ *	in the UTF-8 string.
+ *
+ * Results:
+ *	As above.
+ *
+ * Side effects:
+ *	None.
+ *
+ *---------------------------------------------------------------------------
+ */
+
+char *
+Tcl_UtfAtIndex(src, index)
+    register CONST char *src;	/* The UTF-8 string. */
+    register int index;		/* The position of the desired character. */
+{
+    return (char *) src + index;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tcl_UtfToUpper --
+ *
+ *	Convert lowercase characters to uppercase characters in a UTF
+ *	string in place.  The conversion may shrink the UTF string.
+ *
+ * Results:
+ *	Returns the number of bytes in the resulting string
+ *	excluding the trailing null.
+ *
+ * Side effects:
+ *	Writes a terminating null after the last converted character.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+Tcl_UtfToUpper(str)
+    char *str;			/* String to convert in place. */
+{
+    int len = strlen(str);
+    int i;
+    for (i = 0; i < len; i++) {
+	str[i] = toupper(UCHAR(str[i]));
+    }
+    return len;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tcl_UtfToLower --
+ *
+ *	Convert uppercase characters to lowercase characters in a UTF
+ *	string in place.  The conversion may shrink the UTF string.
+ *
+ * Results:
+ *	Returns the number of bytes in the resulting string
+ *	excluding the trailing null.
+ *
+ * Side effects:
+ *	Writes a terminating null after the last converted character.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+Tcl_UtfToLower(str)
+    char *str;			/* String to convert in place. */
+{
+    int len = strlen(str);
+    int i;
+    for (i = 0; i < len; i++) {
+	str[i] = tolower(UCHAR(str[i]));
+    }
+    return len;
+}
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tcl_UtfToTitle --
+ *
+ *	Changes the first character of a UTF string to title case or
+ *	uppercase and the rest of the string to lowercase.  The
+ *	conversion happens in place and may shrink the UTF string.
+ *
+ * Results:
+ *	Returns the number of bytes in the resulting string
+ *	excluding the trailing null.
+ *
+ * Side effects:
+ *	Writes a terminating null after the last converted character.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+Tcl_UtfToTitle(str)
+    char *str;			/* String to convert in place. */
+{
+    int len = strlen(str);
+    int i;
+    if (len>0) {
+	str[0] = toupper(UCHAR(str[i]));
+    }
+    for (i = 1; i < len; i++) {
+	str[i] = tolower(UCHAR(str[i]));
+    }
+    return len;
+
+}
