@@ -4,16 +4,50 @@
  *	Contains platform specific test commands for the Unix platform.
  *
  * Copyright (c) 1996 Sun Microsystems, Inc.
+ * Copyright (c) 1998 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclUnixTest.c 1.5 97/10/31 17:23:42
+ * RCS: @(#) $Id: tclUnixTest.c,v 1.9 1998/10/14 00:32:55 rjohnson Exp $
  */
 
 #include "tclInt.h"
 #include "tclPort.h"
 
+/*
+ * If using stub macro's, make sure that 4 internal definitions
+ * are available. They are not defined in tcl.h or tclInt.h,
+ * because they are not meant to be used anywhere else than
+ * in this test.
+ */
+
+#ifdef TCL_USE_STUBS
+
+Tcl_StubTable *tclStubTable;
+#define TclpCloseFile(file) \
+	(tclStubTable->closeFile(file))
+#define TclpCreatePipe(readPipe, writePipe) \
+	(tclStubTable->createPipe((VOID *) (readPipe), (VOID *) (writePipe)))
+#define TclUnixWaitForFile(fd, mask, timeout) \
+	(tclStubTable->unixWaitForFile(fd, mask, timeout))
+#define tclExecutableName \
+	(*(tclStubTable->executableName))
+
+#else /* TCL_USE_STUBS */
+
+#define Tcl_Required(a,b,c) Tcl_PkgRequire(a,"Tcl",b,c)
+
+#endif /* TCL_USE_STUBS */
+
+/*
+ * The headers are needed for the testalarm command that verifies the
+ * use of SA_RESTART in signal handlers.
+ */
+
+#include <signal.h>
+#include <sys/resource.h>
+
 /*
  * The following macros convert between TclFile's and fd's.  The conversion
  * simple involves shifting fd's up by one to ensure that no valid fd is ever
@@ -48,6 +82,12 @@ typedef struct Pipe {
 #define MAX_PIPES 10
 static Pipe testPipes[MAX_PIPES];
 
+/*
+ * The stuff below is used by the testalarm and testgotsig ommands.
+ */
+
+static char *gotsig = "0";
+
 /*
  * Forward declarations of procedures defined later in this file:
  */
@@ -64,7 +104,11 @@ static int		TestgetopenfileCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, char **argv));
 int			TclplatformtestInit _ANSI_ARGS_((Tcl_Interp *interp));
 int			Unixtest_Init _ANSI_ARGS_((Tcl_Interp *interp));
-
+static int		TestalarmCmd _ANSI_ARGS_((ClientData dummy,
+			    Tcl_Interp *interp, int argc, char **argv));
+static int		TestgotsigCmd _ANSI_ARGS_((ClientData dummy,
+			    Tcl_Interp *interp, int argc, char **argv));
+static void 		AlarmHandler _ANSI_ARGS_((int));
 
 /*
  *----------------------------------------------------------------------
@@ -121,9 +165,18 @@ Unixtest_Init(interp)
     Tcl_Interp *interp;		/* Interpreter to add commands to. */
 #endif
 {
-    if (Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 1) == NULL) {
+    if (Tcl_Required(interp, TCL_VERSION, 1) == NULL) {
 	return TCL_ERROR;
     }
+#ifdef TCL_USE_STUBS
+    if (tclStubTable->magic != TCL_STUB_MAGIC ||
+	    tclStubTable->magic2 != TCL_STUB_MAGIC) {
+	Tcl_AppendResult(interp, "Cannot execute test: ",
+		"Incompatible stub table in Tcl\n",
+		"Please recompile the UNIX test function", 0);
+	return TCL_ERROR;
+    }
+#endif
     if (Tcl_PkgProvide(interp, "Unixtest", TCL_VERSION) != TCL_OK) {
         return TCL_ERROR;
     }
@@ -135,6 +188,10 @@ Unixtest_Init(interp)
             (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateCommand(interp, "testgetopenfile", TestgetopenfileCmd,
             (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
+    Tcl_CreateCommand(interp, "testalarm", TestalarmCmd,
+            (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
+    Tcl_CreateCommand(interp, "testgotsig", TestgotsigCmd,
+            (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
     return TCL_OK;
 }
 
@@ -400,11 +457,19 @@ TestFileHandlerProc(clientData, mask)
  */
 
 static int
+#ifdef _USING_PROTOTYPES_
+TestfilewaitCmd (
+    ClientData clientData,		/* Not used. */
+    Tcl_Interp *interp,			/* Current interpreter. */
+    int argc,				/* Number of arguments. */
+    char **argv)			/* Argument strings. */
+#else
 TestfilewaitCmd(clientData, interp, argc, argv)
     ClientData clientData;		/* Not used. */
     Tcl_Interp *interp;			/* Current interpreter. */
     int argc;				/* Number of arguments. */
     char **argv;			/* Argument strings. */
+#endif
 {
     int mask, result, timeout;
     Tcl_Channel channel;
@@ -469,11 +534,19 @@ TestfilewaitCmd(clientData, interp, argc, argv)
  */
 
 static int
+#ifdef _USING_PROTOTYPES_
+TestfindexecutableCmd (
+    ClientData clientData,		/* Not used. */
+    Tcl_Interp *interp,			/* Current interpreter. */
+    int argc,				/* Number of arguments. */
+    char **argv)			/* Argument strings. */
+#else
 TestfindexecutableCmd(clientData, interp, argc, argv)
     ClientData clientData;		/* Not used. */
     Tcl_Interp *interp;			/* Current interpreter. */
     int argc;				/* Number of arguments. */
     char **argv;			/* Argument strings. */
+#endif
 {
     char *oldName;
 
@@ -545,3 +618,131 @@ TestgetopenfileCmd(dummy, interp, argc, argv)
     }
     return TCL_OK;
 }
+
+/*
+ *----------------------------------------------------------------------
+ * TestalarmCmd --
+ *
+ *	Test that EINTR is handled correctly by generating and
+ *	handling a signal.  This requires using the SA_RESTART
+ *	flag when registering the signal handler.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Sets up an signal and async handlers.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int
+#ifdef _USING_PROTOTYPES_
+TestalarmCmd (
+    ClientData clientData,		/* Not used. */
+    Tcl_Interp *interp,			/* Current interpreter. */
+    int argc,				/* Number of arguments. */
+    char **argv)			/* Argument strings. */
+#else
+TestalarmCmd(clientData, interp, argc, argv)
+    ClientData clientData;		/* Not used. */
+    Tcl_Interp *interp;			/* Current interpreter. */
+    int argc;				/* Number of arguments. */
+    char **argv;			/* Argument strings. */
+#endif
+{
+#ifdef SA_RESTART
+    unsigned int sec;
+    struct sigaction action;
+
+    if (argc > 1) {
+	Tcl_GetInt(interp, argv[1], (int *)&sec);
+    } else {
+	sec = 1;
+    }
+
+    /*
+     * Setup the signal handling that automatically retries
+     * any interupted I/O system calls.
+     */
+    action.sa_handler = AlarmHandler;
+    memset((void *)&action.sa_mask, 0, sizeof(sigset_t));
+    action.sa_flags = SA_RESTART;
+
+    if (sigaction(SIGALRM, &action, NULL) < 0) {
+	Tcl_AppendResult(interp, "sigaction: ", Tcl_PosixError(interp), NULL);
+	return TCL_ERROR;
+    }
+    if (alarm(sec) < 0) {
+	Tcl_AppendResult(interp, "alarm: ", Tcl_PosixError(interp), NULL);
+	return TCL_ERROR;
+    }
+    return TCL_OK;
+#else
+    Tcl_AppendResult(interp, "warning: sigaction SA_RESTART not support on this platform", NULL);
+    return TCL_ERROR;
+#endif
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * AlarmHandler --
+ *
+ *	Signal handler for the alarm command.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ * 	Calls the Tcl Async handler.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+#ifdef _USING_PROTOTYPES_
+AlarmHandler(
+    int dummy)
+#else
+AlarmHandler(dummy)
+    int dummy;
+#endif
+{
+    gotsig = "1";
+}
+
+/*
+ *----------------------------------------------------------------------
+ * TestgotsigCmd --
+ *
+ * 	Verify the signal was handled after the testalarm command.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Resets the value of gotsig back to '0'.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int
+#ifdef _USING_PROTOTYPES_
+TestgotsigCmd(
+    ClientData clientData,		/* Not used. */
+    Tcl_Interp *interp,			/* Current interpreter. */
+    int argc,				/* Number of arguments. */
+    char **argv)			/* Argument strings. */
+#else
+TestgotsigCmd(clientData, interp, argc, argv)
+    ClientData clientData;		/* Not used. */
+    Tcl_Interp *interp;			/* Current interpreter. */
+    int argc;				/* Number of arguments. */
+    char **argv;			/* Argument strings. */
+#endif
+{
+    Tcl_AppendResult(interp, gotsig, (char *) NULL);
+    gotsig = "0";
+    return TCL_OK;
+}
