@@ -8,7 +8,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclWinInt.h 1.7 97/06/25 10:56:14
+ * RCS: @(#) $Id: tclWinInt.h,v 1.5 1998/09/14 18:40:20 stanton Exp $
  */
 
 #ifndef _TCLWININT
@@ -21,6 +21,11 @@
 #include "tclPort.h"
 #endif
 
+#ifdef BUILD_tcl
+# undef TCL_STORAGE_CLASS
+# define TCL_STORAGE_CLASS DLLEXPORT
+#endif
+
 /*
  * Some versions of Borland C have a define for the OSVERSIONINFO for
  * Win32s and for NT, but not for Windows 95.
@@ -32,8 +37,11 @@
 
 EXTERN int		TclWinGetPlatformId(void);
 EXTERN void		TclWinInit(HINSTANCE hInst);
+EXTERN TclFile		TclWinMakeFile _ANSI_ARGS_((HANDLE hFile));
 EXTERN int		TclWinSynchSpawn(void *args, int type, void **trans,
 				      Tcl_Pid *pidPtr);
 
+# undef TCL_STORAGE_CLASS
+# define TCL_STORAGE_CLASS DLLIMPORT
 
 #endif	/* _TCLWININT */
