@@ -10,32 +10,40 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclWinReg.c 1.8 97/08/01 11:17:49
+ * RCS: @(#) $Id: tclWinReg.c,v 1.6 1998/09/14 18:40:20 stanton Exp $
  */
 
 #include <tcl.h>
 #include <stdlib.h>
 
+#ifdef TCL_USE_STUBS
+Tcl_StubTable *tclStubTable;
+#else
+#define Tcl_Required(a,b,c) Tcl_PkgRequire(a,"Tcl",b,c)
+#endif
+
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #undef WIN32_LEAN_AND_MEAN
 
+/*
+ * TCL_STORAGE_CLASS is set unconditionally to DLLEXPORT because the
+ * Registry_Init declaration is in the source file itself, which is only
+ * accessed when we are building a library.
+ */
+
+#undef TCL_STORAGE_CLASS
+#define TCL_STORAGE_CLASS DLLEXPORT
+
 /*
  * VC++ has an alternate entry point called DllMain, so we need to rename
  * our entry point.
  */
 
-#ifndef STATIC_BUILD
-#if defined(_MSC_VER)
-#   define EXPORT(a,b) __declspec(dllexport) a b
-#   define DllEntryPoint DllMain
-#else
-#   if defined(__BORLANDC__)
-#	define EXPORT(a,b) a _export b
-#   else
-#	define EXPORT(a,b) a b
-#   endif
-#endif
+#ifdef DLL_BUILD
+# if defined(_MSC_VER) || defined(__GNUC__)
+#  define DllEntryPoint DllMain
+# endif
 #endif
 
 /*
@@ -59,7 +67,8 @@
 
 static char *rootKeyNames[] = {
     "HKEY_LOCAL_MACHINE", "HKEY_USERS", "HKEY_CLASSES_ROOT",
-    "HKEY_CURRENT_USER", "HKEY_CURRENT_CONFIG", NULL
+    "HKEY_CURRENT_USER", "HKEY_CURRENT_CONFIG",
+    "HKEY_PERFORMANCE_DATA", "HKEY_DYN_DATA", NULL
 };
 
 static HKEY rootKeys[] = {
@@ -114,7 +123,7 @@ static int		SetValue(Tcl_Interp *interp, Tcl_Obj *keyNameObj,
 			    Tcl_Obj *valueNameObj, Tcl_Obj *dataObj,
 			    Tcl_Obj *typeObj);
 
-EXTERN EXPORT(int,Registry_Init)(Tcl_Interp *interp);
+EXTERN int Registry_Init(Tcl_Interp *interp);
 
 /*
  *----------------------------------------------------------------------
@@ -136,7 +145,7 @@ EXTERN EXPORT(int,Registry_Init)(Tcl_Interp *interp);
  */
 
 #ifdef __WIN32__
-#ifndef STATIC_BUILD
+#ifdef DLL_BUILD
 BOOL APIENTRY
 DllEntryPoint(
     HINSTANCE hInst,		/* Library instance handle. */
@@ -164,9 +173,13 @@ DllEntryPoint(
  *----------------------------------------------------------------------
  */
 
-EXPORT(int,Registry_Init)(
+int
+Registry_Init(
     Tcl_Interp *interp)
 {
+    if (Tcl_Required(interp, TCL_VERSION, 0) == NULL) {
+    return TCL_ERROR;
+    }
     Tcl_CreateObjCommand(interp, "registry", RegistryObjCmd, NULL, NULL);
     return Tcl_PkgProvide(interp, "registry", "1.0");
 }
